// Copyright (c) 2021, Philipp Matthias Schäfer (philipp.matthias.schaefer@posteo.de)
//
// This file is part of the WebLDAPPasswd application.
//
// The WebLDAPPasswd application is free software: you can redistribute it
// and/or modify it under the terms of the GNU Affero General Public License as
// published by the Free Software Foundation, either version 3 of the License or
// (at your option) any later version.
//
// The WebLDAPPasswd application is distributed in the hope that it will be
// useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
// General Public License for more details.
//
// You should have received a copy of the GNU General Affero Public License
// along with the WebLDAPPasswd. If not, see <https://www.gnu.org/licenses/>.
#![feature(proc_macro_hygiene, decl_macro)]

mod api;
mod config;
mod context;
mod r#static;

use anyhow::{Context, Result};
use clap::{Arg, App, crate_version};
use rocket::routes;
use rocket::config::{Config, Environment};

const DEFAULT_CONFIG_FILE_PATH: &str = "/etc/webldappasswd/config.json";

fn main() -> Result<()> {
    let matches = App::new("WebLDAPPasswd")
        .version(crate_version!())
        .arg(Arg::with_name("config")
             .short("c")
             .long("config")
             .value_name("CONFIG_FILE_PATH")
             .help("Custom configuration file")
             .takes_value(true))
        .get_matches();

    let config_file_path = matches
        .value_of("config")
        .unwrap_or(DEFAULT_CONFIG_FILE_PATH);

    let config = config::load(config_file_path)?;

    let context = context::Context::new(&config)?;

    let rocket_config = Config::build(Environment::Production)
        .address(&config.host)
        .port(config.port)
        .finalize()
        .with_context(|| format!(
            "Bad host address ({}) in configuration file '{}'",
            config.host,
            config_file_path
        ))?;

    rocket::custom(rocket_config)
        .mount("/", routes![r#static::index,
                            r#static::legal,
                            r#static::css,
                            r#static::js,
                            r#static::checkmark,
                            r#static::cross,
                            r#static::logo,
                            r#static::hourglass,
                            api::update])
        .manage(context)
        .launch();

    Ok(())
}
